# Ruby Cron Parser
# Copyright (C) 2009 Filipe Felisberto
# Developed for Amahi Home Server
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License v3
# (29 June 2007), as published in the COPYING file.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# file COPYING for more details.
require 'rubygems'
require '/var/hda/platform/html/config/environment'
require '/var/hda/platform/apps/servertweet/amahi_notificator.rb'
require 'syslog'

class RubyCron
    @calendar
    def initialize (str)
        test = str[/\A(((\d{1,2}([,|-]\d{1,2})*)|[*])\s){4}((\d{1,2}([,|-]\d{1,2})*)|[*]){1}\z/] != nil
        
        if test then
            values = str.split(" ")
            @calendar = Array.new(5)
            error = RubyCron.create_calendar(values,@calendar)
            if(error != "") then
                raise RuntimeError,error
            end
        else
            raise RuntimeError,"String Badly Formed"
        end
    end
    
    def self.validate_string(str)
        test = str[/\A(((\d{1,2}([,|-]\d{1,2})*)|[*])\s){4}((\d{1,2}([,|-]\d{1,2})*)|[*]){1}\z/] != nil
        
        if test then
            values = str.split(" ")
            arrays = Array.new(5)
            error = create_calendar(values,arrays)
            return error
        else
            return "String Badly Formed"
        end
        return ""
    end
    
    def next_event()
    	present = Time.new
    	future = Time.new
    	
      year = present.year
    	month = present.month
    	day = present.day
    	hour = present.hour
    	min = present.min

    	equal = true
    	equal_week = true

    	min = get_next(@calendar[0],present.min,false) 
    	if(min <= present.min) then
    			equal = false
    	end

    	hour = get_next(@calendar[1],present.hour,equal)
    	if(@calendar[0].size == 60 and hour != present.hour) then
			min=0		
		end
    	if(hour < present.hour or (hour == present.hour and !equal)) then
    			equal =false
    			equal_week = false
    	else
    			equal = true
    	end

    	if(@calendar[4] == "*" or @calendar[2].size != 31) then
    		p_day = present.day
    		change_m= false
    		start_year=year
    		begin
    			day = get_next(@calendar[2],p_day,equal )        
    			if(day < present.day or (day == present.day and !equal) or change_m ) then
    					equal =false
    			else
    					equal = true
    			end
    			month = get_next(@calendar[3],present.month,equal)
    			if(month < present.month or (month == present.month and !equal)) then
    				year +=1
    				if start_year+5 < year then
    					raise RuntimeError,"Bad Cron Line"
    				end
    			end
    			future = Time.local(year,month,day,hour,min)
    			p_day=1
    			change_m=true
    			#puts "future #{future.day} day #{day}"
    		end while (future.day != day)
    		#week test
    		if(@calendar[4] != "*") then                
    			future_week = get_next_day_week(@calendar[4],present,equal_week)
    			if(future > future_week) then
    				future = Time.local(future_week.year,future_week.month,future_week.day,hour,min) 
    			end
    		end
    	else
			future = Time.local(year,month,day,hour,min)
    		#future =  get_next_day_week_with_valid_month(@calendar[4],present,@calendar[3],equal)
			future =  get_next_day_week_with_valid_month(@calendar[4],future,@calendar[3],equal)
		end
		
    	if(future.day != present.day or future.month != present.month or future.year != present.year) then
			if(@calendar[0].size == 60) then
				future = future - future.min.minutes			
			end
			if(@calendar[1].size == 24) then
				future = future - future.hour.hours		
			end    	
    	end
    	
    	
    	return future
    end
    
    private
    
    def self.create_calendar(values,arrays)
        #mins
    	if values[0] == "*" then
    			arrays[0] = (0..59).to_a
    	else
    			arrays[0] =create_array(values[0])
    			if validate_minutes(arrays[0]) == false then
    					return "Minutes with invalid numbers"
    			end
    	end
    	#hours
    	if values[1] == "*" then
    			arrays[1] = (0..23).to_a
    	else
    			arrays[1] =create_array(values[1])
    			if validate_hours(arrays[1]) == false then
    					return "Hours with invalid numbers"
    			end
    	end
    	#days
    	if values[2] == "*" then
    			arrays[2] = (1..31).to_a
    	else
    			arrays[2] =create_array(values[2])
    			if validate_day_month(arrays[2]) == false then
    					return "Days with invalid numbers"
    			end
    	end
    	#month
    	if values[3] == "*" then
    			arrays[3] = (1..12).to_a
    	else
    			arrays[3] =create_array(values[3])
    			if validate_month(arrays[3]) == false then
    					return "Months with invalid numbers"
    			end
    	end
    	#week day
    	if values[4] == "*" then
    			arrays[4] = "*"
    	else
    			arrays[4] =create_array(values[4])
    			if validate_day_week(arrays[4]) == false then
    					return "Week with invalid numbers"
    			end
    	end
    	return ""
    end
    
    def self.create_array(value)
        array = Array.new
        numbers = value.scan(/\d+/)
        #puts numbers
        oper = value.scan(/\D/)
        #puts oper
        i = 0
        while i < numbers.length do
            if(oper[i]=="-") then
                if(i+1<=numbers.length and oper[i+1]=="-") then
                    return nil
                end
                a = (numbers[i].to_i..numbers[i+1].to_i).to_a
                array = array | a
                i += 2
            else
                array << numbers[i].to_i
                i += 1
            end
        end
        return array.uniq.sort     
    end
    def get_next(array,number,equal)

        if(equal == true) then 
            if(array.include?(number)==true) then
                #puts "equal was true #{number}"
                return number
            end
        end

        array.each do |value|
            if(value > number) then
                #puts "Returning #{value}"
                return value

            end
        end
        return array[0]
    end
    def self.validate_minutes(array)
        if(array[0]=="*") then
            return true
        end
        array.each do |value|
            if(value > 59 or value < 0) then
                return false
            end
        end
        return true
    end
    def self.validate_hours(array)
        if(array[0]=="*") then
            return true
        end
        array.each do |value|
            if(value > 23 or value < 0) then
                return false
            end
        end
        return true
    end
    def self.validate_day_month(array)
        if(array[0]=="*") then
            return true
        end
        array.each do |value|
            if(value > 31 or value < 1) then
                return false
            end
        end
        return true
    end
    def self.validate_month(array)
        if(array[0]=="*") then
            return true
        end
        array.each do |value|
            if(value > 12 or value < 1) then
                return false
            end
        end
        return true
    end
    def self.validate_day_week(array)
        if(array[0]=="*") then
            return true
        end
        array.each do |value|
            if(value > 6 or value < 0) then
                return false
            end
        end
        return true
    end
    def get_next_day_week(week,present,equal)
        wday = get_next(week,present.wday,equal)
        
        #puts "WDAY #{wday}"
		if(present.wday == wday and equal)
			return present
		end
		if(wday ==0) then
            wday =7
        end
        days = (present.wday < wday ? wday - present.wday  : 7 - (present.wday-wday))
        #puts "PRESENT WDAY #{present.wday}"
        #puts "PRESENT #{present}"
        #puts "DAYS #{days}"
        future_week = present + days.days
        return future_week
    end

    def get_next_day_week_with_valid_month(week,present,months,equal)
        year = present.year
        min=present.min
        hour = present.hour
        equal_month = true
        time = present
        begin
            month = get_next(months,time.month,equal_month)
            if(month < time.month or (month == time.month and !equal_month)) then
                year +=1
            end
            if(equal_month == false) then
                puts "#{year} #{month} #{hour} #{min}"
                time = Time.local(year,month,1,hour,min)
            end
            
            future_week =  get_next_day_week(week,time,equal)

            puts "time #{time} future_week #{future_week}"
            equal = true
            equal_month = false
            #puts  "#{months.include?(future_week.month)} with month #{future_week.month}"
            #puts future_week
        end while !months.include?(future_week.month)
        return future_week
    end
end
